@@ -1,14 +1,13 @@
 package Test::Smoke::BuildCFG;
 use strict;
 
+# $Id: BuildCFG.pm 299 2003-07-31 21:08:03Z abeltje $
 use vars qw( $VERSION );
-$VERSION = '0.009';
+$VERSION = '0.003';
 
 use Cwd;
 use File::Spec;
 require File::Path;
-use Test::Smoke::LogMixin;
-use Test::Smoke::Util qw( skip_config );
 
 my %CONFIG = (
     df_v      => 0,
@@ -36,7 +35,11 @@ Handle the build configurations
 
 =head1 METHODS
 
-=head2 Test::Smoke::BuildCFG->new( [$cfgname] )
+=over 4
+
+=cut
+
+=item Test::Smoke::BuildCFG->new( [$cfgname] )
 
 [ Constructor | Public ]
 
@@ -66,52 +69,11 @@ sub new {
     $self->read_parse( $config );
 }
 
-=head2 Test::Smoke::BuildCFG->continue( $logfile[, $cfgname, %options] )
+=item Test::Smoke::BuildCFG->config( $key[, $value] )
 
-[Constructor | public]
+[ Accessor | Public ]
 
-Initialize a new object without the configurations that have already
-been fully processed. If *all* configurations have been processed,
-just pass the equivalent of the C<new()> method.
-
-=cut
-
-sub continue {
-    my $proto = shift;
-    my $class = ref $proto ? ref $proto : $proto;
-
-    my $logfile = shift;
-
-    my $self = $class->new( @_ );
-    $self->{_continue} = 1;
-    return $self unless $logfile && -f $logfile;
-
-    my %seen = __get_smoked_configs( $logfile );
-    my @not_seen = ();
-    foreach my $config ( $self->configurations ) {
-        push @not_seen, $config unless exists $seen{ "$config" } ||
-                                       skip_config( $config );
-    }
-    return $self unless @not_seen;
-    $self->{_list} = \@not_seen;
-    return $self;
-}
-
-=head2 $bldcfg->verbose
-
-[ Getter | Public]
-
-Get verbosity.
-
-=cut
-
-sub verbose { $_[0]->{v} }
-
-=head2 Test::Smoke::BuildCFG->config( $key[, $value] )
-
-[ ClassAccessor | Public ]
-
-C<config()> is an interface to the package lexical C<%CONFIG>,
+C<config()> is an interface to the package lexical C<%CONFIG>, 
 which holds all the default values for the C<new()> arguments.
 
 With the special key B<all_defaults> this returns a reference
@@ -139,7 +101,7 @@ sub config {
     return $CONFIG{ "df_$key" };
 }
 
-=head2 $self->read_parse( $cfgname )
+=item $self->read_parse( $cfgname )
 
 C<read_parse()> reads the build configurations file and parses it.
 
@@ -154,7 +116,7 @@ sub read_parse {
     return $self;
 }
 
-=head2 $self->_read( $nameorref )
+=item $self->_read( $nameorref )
 
 C<_read()> is a private method that handles the reading.
 
@@ -190,7 +152,7 @@ sub _read {
         $self->{_list} = $nameorref->{_list};
         $vmsg = "continuing smoke";
     } elsif ( ref $nameorref eq 'GLOB' ) {
-        *BUILDCFG = *$nameorref;
+	*BUILDCFG = *$nameorref;
         $self->{_buildcfg} = do { local $/; <BUILDCFG> };
         $vmsg = "anonymous filehandle";
     } else {
@@ -201,34 +163,33 @@ sub _read {
                 $vmsg = $nameorref;
             } else {
                 require Carp;
-                Carp::carp("Cannot read buildconfigurations ($nameorref): $!");
+                Carp::carp "Cannot read buildconfigurations ($nameorref): $!";
                 $self->{_buildcfg} = $self->default_buildcfg();
                 $vmsg = "internal content";
             }
         } else { # Allow intentional default_buildcfg()
             $self->{_buildcfg} = $self->default_buildcfg();
             $vmsg = "internal content";
-        }
+        } 
     }
-    $vmsg .= "[continue]" if $self->{_continue};
-    $self->log_info("Reading build configurations from %s", $vmsg);
+    $self->{v} and print "Reading build configurations from $vmsg\n";
 }
 
-=head2 $self->_parse( )
+=item $self->_parse( )
 
 C<_parse()> will split the build configurations file in sections.
 Sections are ended with a line that begins with an equals-sign ('=').
 
 There are two types of section
 
-=over
+=over 8
 
 =item B<buildopt-section>
 
 =item B<policy-section>
 
-A B<policy-section> contains a "target-option". This is a build option
-that should be in the ccflags variable in the F<Policy.sh> file
+A B<policy-section> contains a "target-option". This is a build option 
+that should be in the ccflags variable in the F<Policy.sh> file 
 (see also L<Test::Smoke::Policy>) and starts with a (forward) slash ('/').
 
 A B<policy-section> can have only one (1) target-option.
@@ -244,13 +205,12 @@ sub _parse {
 
     $self->{_sections} = [ ];
     my @sections = split m/^=.*\n/m, $self->{_buildcfg};
-    $self->log_debug("Found %d raw-sections", scalar @sections);
+    $self->{v} > 1 and printf "Found %d raw-sections\n", scalar @sections;
 
     foreach my $section ( @sections ) {
-        chomp $section;
         my $index = 0;
-        my %opts = map { s/^\s+$//; $_ => $index++ }
-            grep !/^#/ => split /\n/, $section, -1;
+        my %opts = map { $_ => $index++ } map { s/^\s+$//; $_ }
+            grep !/^#/ => split /\n/, $section;
         # Skip empty sections
         next if (keys %opts == 0) or (exists $opts{ "" } and keys %opts == 1);
 
@@ -265,30 +225,32 @@ sub _parse {
             }
             if ( @targets > 1 ) {
                 require Carp;
-                Carp::carp( "Multiple policy lines in one section:\n\t",
-                            join( "\n\t", @targets ),
-                            "\nWill use /$targets[0]/\n" );
+                Carp::carp "Multiple policy lines in one section:\n\t",
+                           join( "\n\t", @targets ),
+                           "\nWill use /$targets[0]/\n";
             }
-            push @{ $self->{_sections} },
-                 { policy_target => $targets[0],
+            push @{ $self->{_sections} }, 
+                 { policy_target => $targets[0], 
                    args => [ sort {$opts{ $a } <=> $opts{ $b }} keys %opts ] };
 
         } else { # Buildopt section
-            push @{ $self->{_sections} },
+            push @{ $self->{_sections} }, 
                  [ sort {$opts{ $a } <=> $opts{ $b }} keys %opts ];
         }
     }
     # Make sure we have at least *one* section
     push @{ $self->{_sections} }, [ "" ] unless @{ $self->{_sections} };
 
-    $self->log_debug("Left with %d parsed sections", scalar @{$self->{_sections}});
+    $self->{v} > 1 and printf "Left with %d parsed sections\n", 
+                              scalar @{ $self->{_sections} };
     $self->_serialize;
-    $self->log_debug("Found %d (unfiltered) configurations", scalar @{$self->{_list}});
+    $self->{v} > 1 and printf "Found %d (unfiltered) configurations\n", 
+                              scalar @{ $self->{_list} };
 }
 
-=head2 $self->_serialize( )
+=item $self->_serialize( )
 
-C<_serialize()> creates a list of B<Test::Smoke::BuildCFG::Config>
+C<_serialize()> creates a list of B<Test::Smoke::BuildCFG::Config> 
 objects from the parsed sections.
 
 =cut
@@ -302,7 +264,7 @@ sub _serialize {
     $self->{_list} = $list;
 }
 
-=head2 __build_list( $list, $previous_args, $policy_subst, $this_cfg, @cfgs )
+=item __build_list( $list, $previous_args, $policy_subst, $this_cfg, @cfgs )
 
 Recursive sub, mainly taken from the old C<run_tests()> in F<mktest.pl>
 
@@ -322,7 +284,7 @@ sub __build_list {
         $config_args .= " $conf" if length $conf;
 
         my @substitutions = @$policy_subst;
-        push @substitutions, [ $policy_target, $conf ]
+        push @substitutions, [ $policy_target, $conf ] 
             if defined $policy_target;
 
         if ( @cfgs ) {
@@ -336,7 +298,7 @@ sub __build_list {
     }
 }
 
-=head2 $buildcfg->configurations( )
+=item $buildcfg->configurations( )
 
 Returns the list of configurations (Test::Smoke::BuildCFG::Config objects)
 
@@ -348,78 +310,9 @@ sub configurations {
     @{ $self->{_list} };
 }
 
-=head2 $buildcfg->policy_targets( )
+=item Test::Smoke::BuildCFG->default_buildcfg()
 
-Returns a list of policytargets from the policy substitution sections
-
-=cut
-
-sub policy_targets {
-    my $self = shift;
-
-    return unless UNIVERSAL::isa( $self->{_sections}, "ARRAY" );
-
-    my @targets;
-    for my $section ( @{ $self->{_sections} } ) {
-        next unless UNIVERSAL::isa( $section, "HASH" ) &&
-                    $section->{policy_target};
-        push @targets, $section->{policy_target};
-    }
-
-    return @targets;
-}
-
-=head2 as_string
-
-Return the parsed configuration as a string.
-
-=cut
-
-sub as_string {
-    my $self = shift;
-    my @sections;
-    for my $section ( @{ $self->{_sections} } ) {
-        if ( UNIVERSAL::isa( $section, 'ARRAY' ) ) {
-            push @sections, $section;
-        } elsif ( UNIVERSAL::isa( $section, 'HASH' ) ) {
-            push @sections, [
-                "/$section->{policy_target}/",
-                @{ $section->{args} },
-            ];
-        }
-    }
-    return join "=\n", map join( "\n", @$_, "" ) => @sections;
-}
-
-=head2 __get_smoked_configs( $logfile )
-
-Parse the logfile and return a hash(ref) of already processed
-configurations.
-
-=cut
-
-sub __get_smoked_configs {
-    my( $logfile ) = @_;
-
-    my %conf_done = ( );
-    local *LOG;
-    if ( open LOG, "< $logfile" ) {
-        my $conf;
-        # A Configuration is done when we detect a new Configuration:
-        # or the phrase "Finished smoking $patch"
-        while ( <LOG> ) {
-            s/^Configuration:\s*// || /^Finished smoking/ or next;
-            $conf and $conf_done{ $conf }++;
-            chomp; $conf = $_;
-        }
-        close LOG;
-    }
-    return wantarray ? %conf_done : \%conf_done;
-}
-
-=head2 Test::Smoke::BuildCFG->default_buildcfg()
-
-This is a constant that returns a textversion of the default
+This is a constant that returns a textversion of the default 
 configuration.
 
 =cut
@@ -444,23 +337,16 @@ sub default_buildcfg() {
 __EOCONFIG__
 }
 
-=head2 new_configuration( $config )
+1;
 
-A wrapper around C<< Test::Smoke::BuildCFG::Config->new() >> so the
-object is accessible from outside this package.
+=back
 
 =cut
 
-sub new_configuration {
-    return Test::Smoke::BuildCFG::Config->new( @_ );
-}
-
-1;
-
 package Test::Smoke::BuildCFG::Config;
 
 use overload
-    '""'     => sub { $_[0]->[0] || "" },
+    '""'     => sub { $_[0]->[0] },
     fallback => 1;
 
 use Text::ParseWords qw( quotewords );
@@ -477,7 +363,7 @@ or
 
     my $bcfg = Test::Smoke::BuildCFG::Config->new;
     $bcfg->args( $args );
-    $bcfg->policy( [ -DDEBUGGING => '-DDEBUGGING' ],
+    $bcfg->policy( [ -DDEBUGGING => '-DDEBUGGING' ], 
                    [ -DPERL_COPY_ON_WRITE => '' ] );
 
     if ( $bcfg->has_arg( '-Duseithreads' ) ) {
@@ -486,14 +372,16 @@ or
 
 =head1 DESCRIPTION
 
-This is a simple object that holds both the build arguments and the
+This is a simple object that holds both the build arguments and the 
 policy substitutions. The build arguments are stored as a string and
 the policy subtitutions are stored as a list of lists. Each substitution is
 represented as a list with the two elements: the target and its substitute.
 
 =head1 METHODS
 
-=head2 Test::Smoke::BuildCFG::Config->new( [ $args[, \@policy_substs ]] )
+=over 4
+
+=item Test::Smoke::BuildCFG::Config->new( [ $args[, \@policy_substs ]] )
 
 Create the new object as an anonymous list.
 
@@ -511,7 +399,7 @@ sub new {
     $self;
 }
 
-=head2 $buildcfg->args( [$args] )
+=item $buildcfg->args( [$args] )
 
 Accessor for the build arguments field.
 
@@ -528,7 +416,7 @@ sub args {
     $self->[0];
 }
 
-=head2 $buildcfg->policy( [@substitutes] )
+=item $buildcfg->policy( [@substitutes] )
 
 Accessor for the policy substitutions.
 
@@ -538,7 +426,7 @@ sub policy {
     my $self = shift;
 
     if ( @_ ) {
-        my @substitutions = @_ == 1 &&  ref $_[0][0] eq 'ARRAY'
+        my @substitutions = @_ == 1 &&  ref $_[0][0] eq 'ARRAY' 
             ? @{ $_[0] } : @_;
         $self->[1] = \@substitutions;
     }
@@ -546,7 +434,7 @@ sub policy {
     @{ $self->[1] };
 }
 
-=head2 $self->_split_args( )
+=item $self->_split_args( )
 
 Create a hash with all the build arguments as keys.
 
@@ -555,18 +443,14 @@ Create a hash with all the build arguments as keys.
 sub _split_args {
     my $self = shift;
 
-    my $i = 0;
     $self->[2] = {
-        map { ( $_ => $i++ ) } quotewords( '\s+', 1, $self->[0] )
+        map { ( $_ => 1 ) } quotewords( '\s+', 1, $self->[0] )
     };
-    $self->[0] = join( " ", sort {
-        $self->[2]{ $a } <=> $self->[2]{ $b }
-    } keys %{ $self->[2] } ) || "";
 }
 
-=head2 $buildcfg->has_arg( $arg[,...] )
+=item $buildcfg->has_arg( $arg[,...] )
 
-Check the build arguments hash for C<$arg>. If you specify more then one
+Check the build arguments hash for C<$arg>. If you specify more then one 
 the results will be logically ANDed!
 
 =cut
@@ -579,9 +463,9 @@ sub has_arg {
     return $ok;
 }
 
-=head2 $buildcfg->any_arg( $arg[,...] )
+=item $buildcfg->any_arg( $arg[,...] )
 
-Check the build arguments hash for C<$arg>. If you specify more then one
+Check the build arguments hash for C<$arg>. If you specify more then one 
 the results will be logically ORed!
 
 =cut
@@ -594,7 +478,7 @@ sub any_arg {
     return $ok;
 }
 
-=head2 $buildcfg->args_eq( $args )
+=item $buildcfg->args_eq( $args )
 
 C<args_eq()> takes a string of config arguments and returns true if
 C<$self> has exactly the same args as the C<$args> has.
@@ -608,8 +492,8 @@ sub args_eq {
     my $self = shift;
     my $args = shift;
 
-    my $default_args = join "|", sort {
-        length($b) <=> length($a)
+    my $default_args = join "|", sort { 
+        length($b) <=> length($a) 
     } quotewords( '\s+', 1, Test::Smoke::BuildCFG->config( 'dfopts' ) );
 
     my %copy = map { ( $_ => undef ) }
@@ -626,44 +510,10 @@ sub args_eq {
     return (@left || keys %copy) ? 0 : 1;
 }
 
-=head2 $config->rm_arg( $arg[,..] )
-
-Simply remove the argument(s) from the list and recreate the arguments
-line.
-
-=cut
-
-sub rm_arg {
-    my $self = shift;
-
-    foreach my $arg ( @_ ) {
-        exists $self->[2]{ $arg } and delete $self->[2]{ $arg };
-    }
-    $self->[0] = join( " ", sort {
-        $self->[2]{ $a } <=> $self->[2]{ $b }
-    } keys %{ $self->[2] } ) || "";
-}
-
-=head2 $config->vms
-
-Redo the the commandline switches in a VMSish way.
-
-=cut
-
-sub vms {
-    my $self = shift;
-
-    return join( " ", map {
-        tr/"'//d;
-        s/^-//;
-        qq/-"$_"/;
-    } sort {
-        $self->[2]{ $a } <=> $self->[2]{ $b }
-    } keys %{ $self->[2] } ) || "";
-}
-
 1;
 
+=back
+
 =head1 SEE ALSO
 
 L<Test::Smoke::Smoker>, L<Test::Smoke::Syncer::Policy>
