@@ -1,8 +1,9 @@
 package Test::Smoke::SourceTree;
 use strict;
 
+# $Id: SourceTree.pm 231 2003-07-15 03:10:56Z abeltje $
 use vars qw( $VERSION @EXPORT_OK %EXPORT_TAGS );
-$VERSION = '0.002';
+$VERSION = '0.004';
 
 use File::Spec;
 use File::Find;
@@ -149,10 +150,10 @@ sub check_MANIFEST {
     open MANIFEST, "< $manifile" or 
         croak( "Can't open '$manifile': $!" );
 
-    my %manifest = ( ".patch" => ST_MISSING, map { 
-        s/\s.*$//;
-        ( $_ => ST_MISSING );
-    } <MANIFEST> );
+    my %manifest = map { 
+        m|(\S+)|;
+        ( $1 => ST_MISSING );
+    } <MANIFEST>, ".patch", @_;
     close MANIFEST;
 
     # Walk the tree, remove all found files from %manifest
@@ -181,7 +182,7 @@ not declared in the B<MANIFEST> file.
 sub clean_from_MANIFEST {
     my $self = shift;
 
-    my $mani_check = $self->check_MANIFEST;
+    my $mani_check = $self->check_MANIFEST( @_ );
     my @to_remove = grep $mani_check->{ $_ } == ST_UNDECLARED 
         => keys %$mani_check;
 
@@ -220,7 +221,7 @@ sub copy_from_MANIFEST {
     $verbose and printf " %d items OK\n", scalar @manifest_files;
 
     my $dest = $self->new( $dest_dir );
-    File::Path::mkpath( $$dest, $verbose ) unless -d;
+    File::Path::mkpath( $$dest, $verbose ) unless -d $$dest;
 
     require File::Basename;
     require File::Copy;
@@ -233,7 +234,10 @@ sub copy_from_MANIFEST {
         File::Path::mkpath( $dest_path, $verbose ) unless -d $dest_path;
 
         $verbose > 1 and print "$file -> $dest_name ";
-        my $ok = File::Copy::copy( $self->mani2abs( $file ), $dest_name );
+        my $mode = ( stat $self->mani2abs( $file ) )[2] & 07777;
+        -f $dest_name and unlink $dest_name;
+        my $ok = File::Copy::syscopy( $self->mani2abs( $file ), $dest_name );
+        $ok and $ok &&= chmod $mode, $dest_name;
         $ok or carp "copy '$file' ($dest_path): $!\n";
         $ok && $verbose > 1 and print "OK\n";
     }
