@@ -1,11 +1,12 @@
 package Test::Smoke::Syncer;
 use strict;
 
+# $Id: Syncer.pm 322 2003-08-03 15:33:36Z abeltje $
 use vars qw( $VERSION );
-$VERSION = '0.007';
+$VERSION = '0.008';
 
-use File::Spec;
 use Cwd;
+use File::Spec;
 require File::Path;
 
 my %CONFIG = (
@@ -90,9 +91,12 @@ to get your perl source-tree up to date.
 
 =item snapshot
 
-This method uses the B<Net::FTP> module to get the latest snapshot. 
+This method uses the B<Net::FTP> or the B<LWP> module to get the 
+latest snapshot. When the B<server> attribute starts with I<http://>
+the fetching is done by C<LWP::Simple::mirror()>.
 To emulate the C<< rsync --delete >> effect, the current source-tree
 is removed.
+
 The snapshot tarball is handled by either B<tar>/B<gzip> or 
 B<Archive::Tar>/B<Compress::Zlib>.
 
@@ -295,16 +299,24 @@ sub check_dot_patch {
     local *PATCHLEVEL_H;
     my $patchlevel_h = File::Spec->catfile( $self->{ddir}, 'patchlevel.h' );
     if ( open PATCHLEVEL_H, "< $patchlevel_h" ) {
+        my $declaration_seen = 0;
         while ( <PATCHLEVEL_H> ) {
-            /"DEVEL(\d+)"/ or next;
-            $patch_level = $1;
+            $declaration_seen ||= /local_patches\[\]/;
+            $declaration_seen && /^\s+,"(?:DEVEL|MAINT)(\d+)|(RC\d+)"/ or next;
+            $patch_level = $1 || $2 || '?????';
+            if ( $patch_level =~ /^RC/ ) {
+                $patch_level = $self->version_from_patchlevel_h .
+                               "-$patch_level";
+            } else {
+                $patch_level++;
+            }
         }
         # save 'patchlevel.h' mtime, so you can set it on '.patch'
         my $mtime = ( stat PATCHLEVEL_H )[9];
         close PATCHLEVEL_H;
         # Now create '.patch' and return if $patch_level
         # The patchlevel is off by one in snapshots
-        if ( ++$patch_level ) {
+        if ( $patch_level && $patch_level !~ /-RC\d+$/ ) {
             if ( open DOTPATCH, "> $dot_patch" ) {
                 print DOTPATCH "$patch_level\n";
                 close DOTPATCH; # no use generating the error
@@ -312,11 +324,41 @@ sub check_dot_patch {
             }
             $self->{patchlevel} = $patch_level;
             return $self->{patchlevel};
+        } else {
+            $self->{patchlevel} = $patch_level;
+            return $self->{patchlevel}
         }
     }
     return undef;
 }
 
+=item version_from_patchlevel_h( $ddir )
+
+C<version_from_patchlevel_h()> returns a "dotted" version as derived 
+from the F<patchlevel.h> file in the distribution.
+
+=cut
+
+sub version_from_patchlevel_h {
+    my $self = shift;
+
+    my $file = File::Spec->catfile( $self->{ddir}, 'patchlevel.h' );
+
+    my( $revision, $version, $subversion ) = qw( 5 ? ? );
+    local *PATCHLEVEL;
+    if ( open PATCHLEVEL, "< $file" ) {
+        my $patchlevel = do { local $/; <PATCHLEVEL> };
+        close PATCHLEVEL;
+        $revision   = $patchlevel =~ /^#define PERL_REVISION\s+(\d+)/m 
+                    ? $1 : '?';
+        $version    = $patchlevel =~ /^#define PERL_VERSION\s+(\d+)/m
+                    ? $1 : '?';
+        $subversion = $patchlevel =~ /^#define PERL_SUBVERSION\s+(\d+)/m 
+                    ? $1 : '?';
+    }
+    return "$revision.$version.$subversion";
+}
+ 
 =item $syncer->clean_from_directory( $source_dir[, @leave_these] )
 
 C<clean_from_directory()> uses File::Find to get the contents of
@@ -493,13 +535,17 @@ sub sync {
 
 =item $syncer->_fetch_snapshot( )
 
-This does the actual ftpsession.
+C<_fetch_snapshot()> checks to see if 
+C<< S<< $self->{server} =~ m|^https?://| >> && $self->{sfile} >>.
+If so let B<LWP::Simple> do the fetching else do the FTP thing.
 
 =cut
 
 sub _fetch_snapshot {
     my $self = shift;
 
+    return $self->_fetch_snapshot_HTTP if $self->{server} =~ m|^https?://|i;
+
     require Net::FTP;
     my $ftp = Net::FTP->new( $self->{server}, Debug => 0 ) or do {
         require Carp;
@@ -552,6 +598,44 @@ sub _fetch_snapshot {
     return $local_snap;
 }
 
+=item $syncer->_fetch_snapshot_HTTP( )
+
+C<_fetch_snapshot_HTTP()> simply invokes C<< LWP::Simple::mirror() >>.
+
+=cut
+
+sub _fetch_snapshot_HTTP {
+    my $self = shift;
+
+    require LWP::Simple;
+    my $snap_name = $self->{sfile};
+
+    unless ( $snap_name ) {
+        require Carp;
+        Carp::carp "No snapshot specified for $self->{server}$self->{sdir}";
+        return undef;
+    }
+
+    my $local_snap = File::Spec->catfile( $self->{ddir},
+                                          File::Spec->updir, $snap_name );
+    $local_snap = File::Spec->canonpath( $local_snap );
+
+    my $remote_snap = "$self->{server}$self->{sdir}/$self->{sfile}";
+
+    $self->{v} and print "LWP::Simple::mirror($remote_snap)";
+    my $result = LWP::Simple::mirror( $remote_snap, $local_snap );
+    if ( LWP::Simple::is_success( $result ) ) {
+        $self->{v} and print " OK\n";
+        return $local_snap;
+    } elsif ( LWP::Simple::is_error( $result ) ) {
+        $self->{v} and print " not OK\n";
+        return undef;
+    } else {
+        $self->{v} and print " skipped\n";
+        return $local_snap;
+    }
+}
+
 =item __find_snap_name( $ftp, $snapext )
 
 [Not a method!]
