@@ -10,13 +10,15 @@ use strict;
 
 use Test::Smoke;
 use vars qw($VERSION);
-$VERSION = Test::Smoke->VERSION;
+$VERSION = Test::Smoke->VERSION; 
+# $Id: mkovz.pl 316 2003-08-02 12:28:49Z abeltje $
 
 use File::Spec;
 use Cwd;
 use Text::ParseWords;
 
 use FindBin;
+use lib $FindBin::Bin;
 use lib File::Spec->catdir( $FindBin::Bin, 'lib' );
 use Test::Smoke::Util qw( get_ncpu get_smoked_Config );
 
@@ -24,6 +26,7 @@ my $email = shift || 'daily-build-reports@perl.org';
 my $testd = shift || cwd ();
 my $mailer = "/usr/bin/mailx";
 my $locale = shift;
+my $defaultenv = shift;
 
 my %Config = get_smoked_Config( $testd, qw( version cf_email
                                             osname osvers 
@@ -50,6 +53,7 @@ $Config{archname} .= "/$cpus" if $cpus;
 
 my $p_version = sprintf "%d.%03d%03d", split /\./, $Config{version};
 my $is56x = $p_version >= 5.006 && $p_version < 5.007;
+$defaultenv ||= $is56x;
 
 =head1 NAME
 
@@ -95,7 +99,7 @@ It's a hack! It should be picked up from B<mktest.out>
 
 =cut
 
-my @layers = $is56x ? qw( stdio ) : qw( stdio perlio );
+my @layers = $defaultenv ? qw( stdio ) : qw( stdio perlio );
 $locale and push @layers, "locale";
 
 my (%rpt, @confs, %confs, @manifest, $common_cfg);
@@ -119,11 +123,13 @@ $rpt_stat, $rpt_config
 
 open RPT, "> " . File::Spec->catfile ($testd, "mktest.rpt")
     or die "mktest.rpt: $!";
-select RPT;
+my $fh_selected = select RPT;
 
 my $perlio = "";
 my $conf   = "";
 my $debug  = "";
+my %times  = ( start => undef, total => 0 );
+
 $rpt{patch} = "?";
 my ($out, @out) = (File::Spec->catfile ($testd, "mktest.out"), 1 .. 5);
 open OUT, "<$out" or die "Can't open $out: $!";
@@ -136,7 +142,15 @@ for (<OUT>) {
     unshift @out, $_;
     chomp $out[0];
 
-    if (m/^\s*Smoking patch (\d+)/) {
+    if ( my( $status, $time ) = /(Started|Stopped) smoke at (\d+)/ ) {
+        if ( $status eq "Started" ) {
+            $times{start} = $time;
+        } else {
+            $times{total} += ($time - $times{start}) if defined $times{start};
+        }
+        next;
+    }
+    if (m/^\s*Smoking patch (\d+(?:\.\d+\.\d+-RC\d+)?)/) {
         $rpt{patch} = $1;
         next;
     }
@@ -161,10 +175,16 @@ for (<OUT>) {
     }
     if (m/PERLIO\s*=\s*(\w+)/) {
         $perlio = $1;
-#        next;
+        if ( $perlio eq 'minitest' ) {
+            $rpt{$conf}{$debug}{$_} = "-" for @layers;
+            $rpt{$conf}{$debug}{minitest} = "M";
+            $perlio = 'stdio';
+        }
+        # Deal with harness output
+        s/^PERLIO\s*=\s+\w+(?: :crlf)?\s*// and $out[0] = $_;
     }
     if (m/^\s*All tests successful/) {
-        $rpt{$conf}{$debug}{$perlio} = "O";
+        $rpt{$conf}{$debug}{$perlio} = $rpt{$conf}{$debug}{minitest} || "O";
         next;
     }
     if (m/^\s*Skipped this configuration/) {
@@ -193,9 +213,12 @@ for (<OUT>) {
         }
         next;
     }
-    if (m/^\s*Unable to (?=([cbmt]))(?:build|configure|make|test) perl/) {
+    if (my( $status, $mini ) = m/^\s*Unable\ to
+                                 \ (?=([cbmt]))(?:build|configure|make|test)
+                                 \ (anything\ but\ mini)?perl/x) {
+        $mini and $status = uc $status; # M for no perl but miniperl
         foreach my $layer ( @layers ) {
-            $rpt{$conf}{$debug}{ $layer }  = $1;
+            $rpt{$conf}{$debug}{ $layer }  = $status;
         }
         next;
     }
@@ -203,6 +226,8 @@ for (<OUT>) {
     if (m/^\s*FAILED/ || m/^\s*DIED/) {
         foreach my $out (@out) {
             $out =~ m/\.\./ or next;
+            ref $rpt{$conf}{$debug}{$perlio} or
+                $rpt{$conf}{$debug}{$perlio} = []; # Clean up sparse garbage
             push @{$rpt{$conf}{$debug}{$perlio}}, $out . substr $_, 3;
             last;                
         }
@@ -214,21 +239,28 @@ for (<OUT>) {
         push @{$rpt{$conf}{$debug}{$perlio}}, $_;
         next;
     }
+    if ( /^\s+\d+(?:[-\s]+\d+)*/ ) {
+        push @{$rpt{$conf}{$debug}{$perlio}}, $_;
+        next;
+    }
 }
 
-$Config{ccvers}	||= $Config{ccversion} || $Config{gccversion};
+$Config{ccvers}	||= $Config{gccversion} || $Config{ccversion};
 require Sys::Hostname;
 my $this_host = Sys::Hostname::hostname() || "nohost";
+my $this_pver = $^V ? sprintf "%vd", $^V : $];
+$times{showtime} = time_in_hhmm( $times{total} );
+my $time_msg  = $times{showtime} ? " [$times{showtime}]" : "";
 
 print <<EOH;
 Automated smoke report for $Config{version} patch $rpt{patch} on $Config{osname} - $Config{osvers} ($Config{archname})
 ($this_host) using $Config{cc} version $Config{ccvers}
-Report generated by Test::Smoke v$VERSION
+Report by Test::Smoke v$VERSION (perl $this_pver)$time_msg
 
 O = OK  F = Failure(s), extended report at the bottom
 ? = still running or test results not (yet) available
 Build failures during:       - = unknown or N/A
-    c = Configure, m = make, t = make test-prep
+c = Configure, m = make, M = make (after miniperl), t = make test-prep
 
 EOH
 
@@ -245,15 +277,19 @@ $common_cfg = join " ", sort keys %common_args;
 
 $common_cfg ||= 'none';
 
-my %count = ( O => 0, F => 0, m => 0, c => 0, o => 0, t => 0);
+my %count = ( O => 0, F => 0, M => 0, m => 0, c => 0, o => 0, t => 0);
 my @fail;
 for my $conf (@confs) {
     ( $rpt_stat, $rpt_config ) = ( "", $conf );
     for my $debug ("", "D") {
 	for my $perlio ( @layers ) {
+            my $bldenv = $perlio eq 'stdio' && 
+                         $rpt{$conf}{$debug}{minitest}
+                ? 'minitest' : $perlio;
+
 	    my $res = $rpt{$conf}{$debug}{$perlio};
 	    if (ref $res) {
-                $rpt_stat .= "F ";
+                $rpt_stat .= $bldenv eq 'minitest' ? "M " : "F ";
 		my $s_conf = $conf;
 		$debug and substr ($s_conf, 0, 0) = "-DDEBUGGING ";
 		if ( $perlio eq "stdio" && ref $rpt{$conf}{$debug}{perlio} 
@@ -262,15 +298,15 @@ for my $conf (@confs) {
 		    # Squeeze stdio/perlio errors together
 		    push @fail, [ "stdio/perlio", $s_conf, $res ];
 		    next;
-		} elsif ( $perlio eq "perlio" && ref $rpt{$conf}{$debug}{stdio}
+		} elsif ( $bldenv eq "perlio" && ref $rpt{$conf}{$debug}{stdio}
                           && "@{ $rpt{$conf}{$debug}{stdio} }"
                           eq "@{ $rpt{$conf}{$debug}{perlio} }" ) {
                     next;
-                } elsif ( $perlio eq "locale" ) {
+                } elsif ( $bldenv eq "locale" ) {
                     push @fail, [ "locale:$locale", $s_conf, $res ];
                     next;
                 }
-		push @fail, [ $perlio, $s_conf, $res ];
+		push @fail, [ $bldenv, $s_conf, $res ];
 		next;
 	    }
             $rpt_stat .= ( $res ? $res : "?" ) . " ";
@@ -278,15 +314,15 @@ for my $conf (@confs) {
     }
     $rpt_config = join " ", grep defined $_ && !exists $common_args{ $_ }, 
                             quotewords( '\s+', 1, $conf );
-    write;
+    write RPT;
     # special casing the '-' should change PASS-so-far
     # to PASS if the report only has 'O' and '-'
     $count{ $_ }++ for map { 
-        /[OFmct]/ ? $_ : /-/ ? 'O' : 'o'
+        /[OFMmct]/ ? $_ : /-/ ? 'O' : 'o'
     } split ' ', $rpt_stat;
 }
 
-my @rpt_sum_stat = grep $count{ $_ } > 0 => qw( F m c t );
+my @rpt_sum_stat = grep $count{ $_ } > 0 => qw( F M m c t );
 my $rpt_summary = '';
 if ( @rpt_sum_stat ) {
     $rpt_summary = "FAIL(" . join( "", @rpt_sum_stat ) . ")";
@@ -294,7 +330,7 @@ if ( @rpt_sum_stat ) {
     $rpt_summary = $count{o} == 0 ? 'PASS' : 'PASS-so-far';
 }
 
-print $locale ? <<EOL : $is56x ? <<EOS : <<EOE;
+print $locale ? <<EOL : $defaultenv ? <<EOS : <<EOE;
 | | | | | +- LC_ALL = $locale -DDEBUGGING
 | | | | +--- PERLIO = perlio -DDEBUGGING
 | | | +----- PERLIO = stdio  -DDEBUGGING
@@ -329,12 +365,13 @@ $^O,         $rpt_pio,    $rpt_config
                      $rpt_config
 .
 
+    $^ = 'STDOUT_TOP';
     $~ = 'RPT_Fail_Config';
     print "\nFailures:\n\n";
     for my $i (0 .. $#fail) {
         my $ref = $fail[$i];
         ( $rpt_pio, $rpt_config ) = @{ $ref }[0, 1];
-        write;
+        write RPT;
         if ($i < $#fail) { # More squeezing
 	    my $nref = $fail[$i + 1];
             $ref->[0] =~  /\Q$nref->[0]\E/ and
@@ -347,7 +384,7 @@ $^O,         $rpt_pio,    $rpt_config
 @manifest and print RPT "\n\n", @manifest;
 
 close RPT;
-select STDOUT;
+select $fh_selected;
 
 send_mail () unless $email =~ /^no\-?e?mail$/i;
 
@@ -386,6 +423,27 @@ sub win32_username
     return Win32::LoginName() || 'unknown';
 } # win32_username
 
+sub time_in_hhmm {
+    my $diff = shift;
+
+    my $digits = $diff =~ /\./ ? 3 : 0;
+    my $days = int( $diff / (24*60*60) );
+    $diff -= 24*60*60 * $days;
+    my $hour = int( $diff / (60*60) );
+    $diff -= 60*60 * $hour;
+    my $mins = int( $diff / 60 );
+    $diff -=  60 * $mins;
+
+    my @parts;
+    $days and push @parts, sprintf "%d day%s",   $days, $days == 1 ? "" : 's';
+    $hour and push @parts, sprintf "%d hour%s",  $hour, $hour == 1 ? "" : 's';
+    $mins and push @parts, sprintf "%d minute%s",$mins, $mins == 1 ? "" : 's';
+    $diff && !$days && !$hour and
+        push @parts, sprintf "%.${digits}f seconds", $diff;
+
+    return join " ", @parts;
+}
+
 =head1 CHANGES
 
 For more recent changes see B<ChangeLog>.
