@@ -1,15 +1,16 @@
 package TestLib;
 use strict;
 
+# $Id: TestLib.pm 279 2003-07-28 19:48:46Z abeltje $
 use vars qw( $VERSION @EXPORT );
-use base 'Exporter';
-$VERSION = '0.01';
+$VERSION = '0.02';
 
+use base 'Exporter';
 @EXPORT = qw( 
     &whereis 
     &find_unzip &do_unzip
     &find_untargz &do_untargz
-    &get_dir &get_file 
+    &get_dir &get_file &put_file
     &rmtree &mkpath
 );
 
@@ -101,6 +102,33 @@ sub get_file {
     return wantarray ? @content : join "", @content;
 }
 
+=item put_file( $content, @path )
+
+The contents of C<@path> are passed to B<< File::Spec->catfile() >>
+
+Writes C<$content> to that file and returns the success/failure.
+
+=cut
+
+sub put_file {
+    my $contents = shift;
+    my $filename = File::Spec->catfile( @_ );
+
+    local *MYFILE;
+    if ( open MYFILE, "> $filename" ) {
+        print MYFILE $contents;
+        close MYFILE or do {
+            warn "Cannot close (@{[cwd]})$filename: $!";
+            return;
+        };
+    } else {
+        warn "Cannot create (@{[cwd]})$filename: $!";
+        return;
+    }
+
+    return 1;
+}
+
 =item rmtree( @_ )
 
 This is B<< File::Path::rmtree() >>.
@@ -194,7 +222,11 @@ sub find_untargz {
     unless ( $uncompress ) {
         eval { require Archive::Tar; };
         unless ( $@ ) {
-            eval { require Compress::Zlib; };
+            if ( $Archive::Tar::VERSION >= 0.99 ) {
+                eval { require IO::Zlib };
+            } else {
+                eval { require Compress::Zlib };
+            }
             $uncompress = 'Archive::Tar' unless $@;
         }
     }
